/*
Copyright (C) 1996-2001 Id Software, Inc.
Copyright (C) 2002-2005 John Fitzgibbons and others

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

//r_alias.c -- alias model rendering

#include "quakedef.h"

extern qboolean mtexenabled; //johnfitz
extern cvar_t r_drawflat, gl_overbright_models, gl_fullbrights; //johnfitz

//up to 16 color translated skins
gltexture_t *playertextures[MAX_SCOREBOARD]; //johnfitz -- changed to an array of pointers

#define NUMVERTEXNORMALS	162

float	r_avertexnormals[NUMVERTEXNORMALS][3] = {
#include "anorms.h"
};

vec3_t	shadevector;

extern vec3_t	lightcolor; //johnfitz -- replaces "float shadelight" for lit support

// precalculated dot products for quantized angles
#define SHADEDOT_QUANT 16
float	r_avertexnormal_dots[SHADEDOT_QUANT][256] =
#include "anorm_dots.h"
;

float	*shadedots = r_avertexnormal_dots[0];


// fenix@io.com: model animation interpolation

         int lastposenum;	//Preach - have to bring this back for the model interpolation code as it stands
							//Maybe can find a neater solution later
         int lastposenum0;

/*
=============
GL_DrawAliasShadow -- johnfitz -- moved some code here from other places
=============
*/
extern	vec3_t			lightspot;

void GL_DrawAliasShadow (aliashdr_t *paliashdr, int posenum)
{
	trivertx_t	*v, *verts;
	vec3_t	point;
	float	s, t, l, height, lheight, an;
	float	*normal;
	int		i, j, list, count, index;
	int		*order;

	GL_DisableMultitexture ();

	glPushMatrix ();
	R_RotateForEntity (currententity);
	glDisable (GL_TEXTURE_2D);
	glEnable (GL_BLEND);
	glColor4f (0,0,0,0.5);

	an = currententity->angles[1]/180*M_PI;
	shadevector[0] = cos(-an);
	shadevector[1] = sin(-an);
	shadevector[2] = 1;
	VectorNormalize (shadevector);

	lheight = currententity->origin[2] - lightspot[2];

	height = 0;
	verts = (trivertx_t *)((byte *)paliashdr + paliashdr->posedata);
	verts += posenum * paliashdr->poseverts;
	order = (int *)((byte *)paliashdr + paliashdr->commands);

	height = -lheight + 1.0; //FIXME: use GL_PolygonOffset instead or manually raising it 1.0?

	//FIXME: orient shadow onto "lightplane" (a global mplane_t*)

	while (1)
	{
		// get the vertex count and primitive type
		count = *order++;
		if (!count)
			break;		// done
		if (count < 0)
		{
			count = -count;
			glBegin (GL_TRIANGLE_FAN);
		}
		else
			glBegin (GL_TRIANGLE_STRIP);

		do
		{
			order += 2;

			// normals and vertexes come from the frame list
			point[0] = verts->v[0] * paliashdr->scale[0] + paliashdr->scale_origin[0];
			point[1] = verts->v[1] * paliashdr->scale[1] + paliashdr->scale_origin[1];
			point[2] = verts->v[2] * paliashdr->scale[2] + paliashdr->scale_origin[2];

			point[0] -= shadevector[0]*(point[2]+lheight);
			point[1] -= shadevector[1]*(point[2]+lheight);
			point[2] = height;
			glVertex3fv (point);

			verts++;
		} while (--count);

		glEnd ();
	}

	rs_aliaspasses += paliashdr->numtris; //johnfitz

	glEnable (GL_TEXTURE_2D);
	glDisable (GL_BLEND);
	glColor3f (1,1,1);
	glPopMatrix ();
}

/*
=============
GL_DrawAliasFrame
=============
*/
void GL_DrawAliasFrame (aliashdr_t *paliashdr, int posenum)
{
	vec3_t 	vertcolor; //johnfitz -- replaces "float l" for lit support
	trivertx_t	*verts;
	int		*commands; //johnfitz -- renamed from "*order" to reduce confusion
	int		count;

	verts = (trivertx_t *)((byte *)paliashdr + paliashdr->posedata);
	verts += posenum * paliashdr->poseverts;
	commands = (int *)((byte *)paliashdr + paliashdr->commands);

	while (1)
	{
		// get the vertex count and primitive type
		count = *commands++;
		if (!count)
			break;		// done

		if (count < 0)
		{
			count = -count;
			glBegin (GL_TRIANGLE_FAN);
		}
		else
			glBegin (GL_TRIANGLE_STRIP);

		do
		{
			//johnfitz -- multitexture
			if(mtexenabled)
			{
				GL_MTexCoord2fFunc (TEXTURE0, ((float *)commands)[0], ((float *)commands)[1]);
				GL_MTexCoord2fFunc (TEXTURE1, ((float *)commands)[0], ((float *)commands)[1]);
			}
			else
				glTexCoord2f (((float *)commands)[0], ((float *)commands)[1]);
			//johnfitz

			commands += 2;

			//johnfitz -- r_drawflat, lit support
			if (r_drawflat_cheatsafe)
			{
				srand(count * (unsigned int) commands);
				glColor3f (rand()%256/255.0, rand()%256/255.0, rand()%256/255.0);
			}
			else if (r_fullbright_cheatsafe || r_lightmap_cheatsafe)
			{
				glColor3f(1,1,1);
			}
			else
			{
				vertcolor[0] = shadedots[verts->lightnormalindex] * lightcolor[0];
				vertcolor[1] = shadedots[verts->lightnormalindex] * lightcolor[1];
				vertcolor[2] = shadedots[verts->lightnormalindex] * lightcolor[2];
				glColor3fv (vertcolor);
			}
			//johnfitz

			glVertex3f (verts->v[0], verts->v[1], verts->v[2]);
			verts++;
		} while (--count);

		glEnd ();
	}

	rs_aliaspasses += paliashdr->numtris; //johnfitz
}

         /*
         =============
         GL_DrawAliasBlendedFrame

         fenix@io.com: model animation interpolation
         =============
         */
         void GL_DrawAliasBlendedFrame (aliashdr_t *paliashdr, int pose1, int pose2, float blend)
         {
			vec3_t 	vertcolor; //johnfitz -- replaces "float l" for lit support
            trivertx_t* verts1;
            trivertx_t* verts2;
            int*        order;
            int         count;
            vec3_t      d;
        
            lastposenum0 = pose1;
            lastposenum  = pose2;
        
            verts1  = (trivertx_t *)((byte *)paliashdr + paliashdr->posedata);
            verts2  = verts1;

            verts1 += pose1 * paliashdr->poseverts;
            verts2 += pose2 * paliashdr->poseverts;

            order = (int *)((byte *)paliashdr + paliashdr->commands);
            
            for (;;)
               {
               // get the vertex count and primitive type
               count = *order++;
        
               if (!count) break;
        
               if (count < 0)
                  {
                  count = -count;
                  glBegin (GL_TRIANGLE_FAN);
                  }
               else
                  {
                  glBegin (GL_TRIANGLE_STRIP);
                  }

               do
                  {

					  			//johnfitz -- multitexture
			if(mtexenabled)
			{
				GL_MTexCoord2fFunc (TEXTURE0, ((float *)order)[0], ((float *)order)[1]);
				GL_MTexCoord2fFunc (TEXTURE1, ((float *)order)[0], ((float *)order)[1]);
			}
			else
				glTexCoord2f (((float *)order)[0], ((float *)order)[1]);
			//johnfitz
                  order += 2;

                  // normals and vertexes come from the frame list
                  // blend the light intensity from the two frames together
                  d[0] = shadedots[verts2->lightnormalindex] -
                         shadedots[verts1->lightnormalindex];

			//johnfitz -- r_drawflat, lit support
			if (r_drawflat_cheatsafe)
			{
				srand(count * (unsigned int) order);
				glColor3f (rand()%256/255.0, rand()%256/255.0, rand()%256/255.0);
			}
			else if (r_fullbright_cheatsafe || r_lightmap_cheatsafe)
			{
				glColor3f(1,1,1);
			}
			else
			{
				vertcolor[0] = (shadedots[verts1->lightnormalindex] + (blend * d[0])) * lightcolor[0];//Preach
				vertcolor[1] = (shadedots[verts1->lightnormalindex] + (blend * d[0])) * lightcolor[1];//Hope I merged these correctly
				vertcolor[2] = (shadedots[verts1->lightnormalindex] + (blend * d[0])) * lightcolor[2];//Yeah
				glColor3fv (vertcolor);
			}
			//johnfitz

                  VectorSubtract(verts2->v, verts1->v, d);

                  // blend the vertex positions from each frame together
                  glVertex3f (
                             verts1->v[0] + (blend * d[0]),
                             verts1->v[1] + (blend * d[1]),
                             verts1->v[2] + (blend * d[2]));

                  verts1++;
                  verts2++;
                  } while (--count);
               glEnd ();
               }
			
	rs_aliaspasses += paliashdr->numtris;	//johnfitz
											//preach - dunno what this does, but I think it still belongs and works
											//maybe for r_speeds....
         }

/*
=================
R_SetupAliasFrame -- johnfitz -- modified to return posenum instead of calling GL_DrawAliasFrame itself
=================
*/
int R_SetupAliasFrame (aliashdr_t *paliashdr, int frame)
{
	int				posenum, numposes;
	float			interval;

	if ((frame >= paliashdr->numframes) || (frame < 0))
	{
		Con_DPrintf ("R_AliasSetupFrame: no such frame %d\n", frame);
		frame = 0;
	}

	posenum = paliashdr->frames[frame].firstpose;
	numposes = paliashdr->frames[frame].numposes;

	if (numposes > 1)
	{
		interval = paliashdr->frames[frame].interval;
		posenum += (int)(cl.time / interval) % numposes;
	}

	return posenum;
}
         /*
         =================
         R_SetupAliasBlendedFrame

         fenix@io.com: model animation interpolation
         =================
         */
         float R_SetupAliasBlendedFrame (aliashdr_t *paliashdr, int frame, entity_t* e)
			 //Preach - changed the return type to float, this now returns blend, rather than use it straight off
         {
            int   pose;
            int   numposes;
            float blend;

            if ((frame >= paliashdr->numframes) || (frame < 0))
               {
               Con_DPrintf ("R_AliasSetupFrame: no such frame %d\n", frame);
               frame = 0;
               }

            pose = paliashdr->frames[frame].firstpose;
            numposes = paliashdr->frames[frame].numposes;

            if (numposes > 1)
               {
               e->frame_interval = paliashdr->frames[frame].interval;
               pose += (int)(cl.time / e->frame_interval) % numposes;
               }
            else 
               {
               /* One tenth of a second is a good for most Quake animations.
               If the nextthink is longer then the animation is usually meant to pause
               (e.g. check out the shambler magic animation in shambler.qc).  If its
               shorter then things will still be smoothed partly, and the jumps will be
               less noticable because of the shorter time.  So, this is probably a good
               assumption. */
               e->frame_interval = 0.1;
               }

            if (e->pose2 != pose)
               {
               e->frame_start_time = realtime;
               e->pose1 = e->pose2;
               e->pose2 = pose;
               blend = 0;
               }
            else
               {
               blend = (realtime - e->frame_start_time) / e->frame_interval;
               }
      
            // wierd things start happening if blend passes 1
            if (cl.paused || blend > 1) blend = 1;
            
//            GL_DrawAliasBlendedFrame (paliashdr, e->pose1, e->pose2, blend);

	return blend;	//preach - adapting the fitz method
					//the poses are still stored as entity fields so the entities still have to be passed
					//but blend is only calculated once and used as required
         }


/*
=================
R_SetupAliasLighting -- johnfitz -- broken out from R_DrawAliasModel and rewritten
=================
*/
void R_SetupAliasLighting (entity_t	*e)
{
	vec3_t		dist;
	float		add;
	int			i;

	R_LightPoint (e->origin);

	//add dlights
	for (i=0 ; i<MAX_DLIGHTS ; i++)
	{
		if (cl_dlights[i].die >= cl.time)
		{
			VectorSubtract (currententity->origin, cl_dlights[i].origin, dist);
			add = cl_dlights[i].radius - Length(dist);
			if (add > 0)
				VectorMA (lightcolor, add, cl_dlights[i].color, lightcolor);
		}
	}

	// minimum light value on gun (24)
	if (e == &cl.viewent)
	{
		add = 72.0f - (lightcolor[0] + lightcolor[1] + lightcolor[2]);
		if (add > 0.0f)
		{
			lightcolor[0] += add / 3.0f;
			lightcolor[1] += add / 3.0f;
			lightcolor[2] += add / 3.0f;
		}
	}

	// minimum light value on players (8)
	if (currententity > cl_entities && currententity <= cl_entities + cl.maxclients)
	{
		add = 24.0f - (lightcolor[0] + lightcolor[1] + lightcolor[2]);
		if (add > 0.0f)
		{
			lightcolor[0] += add / 3.0f;
			lightcolor[1] += add / 3.0f;
			lightcolor[2] += add / 3.0f;
		}
	}

	// clamp lighting so it doesn't overbright as much (96)
	if (gl_overbright_models.value)
	{
		add = 288.0f / (lightcolor[0] + lightcolor[1] + lightcolor[2]);
		if (add < 1.0f)
			VectorScale(lightcolor, add, lightcolor);
	}

	//hack up the brightness when fullbrights but no overbrights (256)
	if (gl_fullbrights.value && !gl_overbright_models.value)
		if (!strcmp (e->model->name, "progs/flame2.mdl") ||
			!strcmp (e->model->name, "progs/flame.mdl") ||
			!strcmp (e->model->name, "progs/boss.mdl"))
		{
			lightcolor[0] = 256.0f;
			lightcolor[1] = 256.0f;
			lightcolor[2] = 256.0f;
		}

	shadedots = r_avertexnormal_dots[((int)(e->angles[1] * (SHADEDOT_QUANT / 360.0))) & (SHADEDOT_QUANT - 1)];
	VectorScale(lightcolor, 1.0f / 200.0f, lightcolor);
}

/*
=================
R_DrawAliasModel -- johnfitz -- almost completely rewritten
=================
*/
void R_DrawAliasModel (entity_t *e)
{
	aliashdr_t	*paliashdr;
	vec3_t		mins, maxs;
	int			i, anim, posenum;
	float		blend;
	gltexture_t	*tx, *fb;

	VectorAdd (currententity->origin, currententity->model->mins, mins);
	VectorAdd (currententity->origin, currententity->model->maxs, maxs);
	if (R_CullBox (mins, maxs))
		return;

	VectorCopy (currententity->origin, r_entorigin);
	VectorSubtract (r_origin, r_entorigin, modelorg);

	paliashdr = (aliashdr_t *)Mod_Extradata (currententity->model);

	rs_aliaspolys += paliashdr->numtris;

    glPushMatrix ();
		 // fenix@io.com: model transform interpolation
	 if (r_int_transform.value)
		{
		R_BlendedRotateForEntity (e);
		}
	 else
		{
		R_RotateForEntity (e);
		}

	glTranslatef (paliashdr->scale_origin[0], paliashdr->scale_origin[1], paliashdr->scale_origin[2]);
	glScalef (paliashdr->scale[0], paliashdr->scale[1], paliashdr->scale[2]);

	if (gl_smoothmodels.value && !r_drawflat_cheatsafe)
		glShadeModel (GL_SMOOTH);
	if (gl_affinemodels.value)
		glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
	GL_DisableMultitexture();
         // fenix@io.com: model animation interpolation
         if (r_int_animation.value)
            {
      	blend = R_SetupAliasBlendedFrame (paliashdr, currententity->frame, currententity); //Preach, rewritting all this too...
            }																				// needs currententity passed to it
																							// this is different to the original
																							// SetupAliasFrame, not fitz's fault
         else
            {
	posenum = R_SetupAliasFrame (paliashdr, currententity->frame);
            }



	R_SetupAliasLighting (e);

	//
	// set up textures
	//
	anim = (int)(cl.time*10) & 3;
	tx = paliashdr->gltextures[currententity->skinnum][anim];
	fb = paliashdr->fbtextures[currententity->skinnum][anim];
	if (currententity->colormap != vid.colormap && !gl_nocolors.value)
	{
		i = currententity - cl_entities;
		if (i >= 1 && i<=cl.maxclients /* && !strcmp (currententity->model->name, "progs/player.mdl") */)
		    tx = playertextures[i - 1];
	}
	if (!gl_fullbrights.value)
		fb = NULL;

	//
	// draw it
	//
	if (r_drawflat_cheatsafe)
	{
		glDisable (GL_TEXTURE_2D);

	 if (r_int_animation.value)
			  {
			   GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
				}	//Preach, I trust that using currententity makes this work
					//I have a feeling it destroys the point of the fitz reordering of the render code
					//Oh well...
					//Extreme hackage goes on here,
	 else	
		 GL_DrawAliasFrame (paliashdr, posenum);

		glEnable (GL_TEXTURE_2D);
		srand((int) (cl.time * 1000)); //restore randomness
		goto cleanup;
	}

	if (r_fullbright_cheatsafe)
	{
		GL_Bind (tx);
		 if (r_int_animation.value)
			{
			GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
			}//Preach, I trust that using currententity makes this work
		else	
			GL_DrawAliasFrame (paliashdr, posenum);

		goto cleanup;
	}

	if (r_lightmap_cheatsafe)
	{
		glDisable (GL_TEXTURE_2D);
		 if (r_int_animation.value)
			{
			GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
			}//Preach, I trust that using currententity makes this work
		else	
			GL_DrawAliasFrame (paliashdr, posenum);
		glEnable (GL_TEXTURE_2D);
		goto cleanup;
	}

	if (gl_overbright_models.value && !r_drawflat_cheatsafe)
	{
		if  (gl_texture_env_combine && gl_mtexable && fb) //case 1: everything in one pass
		{
			GL_Bind (tx);
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
			glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_MODULATE);
			glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_TEXTURE);
			glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB_EXT, GL_PRIMARY_COLOR_EXT);
			glTexEnvf(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, 2.0f);
			GL_EnableMultitexture(); // selects TEXTURE1
			GL_Bind (fb);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
			glEnable(GL_BLEND);
		 if (r_int_animation.value)
			{
			GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
			}//Preach, I trust that using currententity makes this work
		else	
			GL_DrawAliasFrame (paliashdr, posenum);
			glDisable(GL_BLEND);
			GL_DisableMultitexture();
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		}
		else if (gl_texture_env_combine) //case 2: overbright in one pass, then fullbright pass
		{
		// first pass
			GL_Bind(tx);
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
			glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_MODULATE);
			glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_TEXTURE);
			glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB_EXT, GL_PRIMARY_COLOR_EXT);
			glTexEnvf(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, 2.0f);
		 if (r_int_animation.value)
			{
			GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
			}//Preach, I trust that using currententity makes this work
		else	
			GL_DrawAliasFrame (paliashdr, posenum);
			glTexEnvf(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, 1.0f);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		// second pass
			if (fb)
			{
				GL_Bind(fb);
				glEnable(GL_BLEND);
				glDepthMask(GL_FALSE);
		 if (r_int_animation.value)
			{
			GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
			}//Preach, I trust that using currententity makes this work
		else	
				GL_DrawAliasFrame (paliashdr, posenum);
				glDepthMask(GL_TRUE);
				glDisable(GL_BLEND);
			}
		}
		else //case 3: overbright in two passes, then fullbright pass
		{
		// first pass
			GL_Bind(tx);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			 if (r_int_animation.value)
				{
				GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
				}//Preach, I trust that using currententity makes this work
			else	
				GL_DrawAliasFrame (paliashdr, posenum);
		// second pass
			glEnable(GL_BLEND);
			glBlendFunc (GL_ONE, GL_ONE);
			glDepthMask(GL_FALSE);
			 if (r_int_animation.value)
				{
				GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
				}//Preach, I trust that using currententity makes this work
			else
				GL_DrawAliasFrame (paliashdr, posenum);
			glDepthMask(GL_TRUE);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glDisable(GL_BLEND);
		// third pass
			if (fb)
			{
				GL_Bind(fb);
				glEnable(GL_BLEND);
				glDepthMask(GL_FALSE);
				 if (r_int_animation.value)
					{
					GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
					}//Preach, I trust that using currententity makes this work
				else	
						GL_DrawAliasFrame (paliashdr, posenum);
				glDepthMask(GL_TRUE);
				glDisable(GL_BLEND);
			}
		}
	}
	else
	{
		if (!fb) //case 4: no fullbright mask
		{
			GL_Bind(tx);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			 if (r_int_animation.value)
				{
				GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
				}//Preach, I trust that using currententity makes this work
			else	
				GL_DrawAliasFrame (paliashdr, posenum);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		}
		else if (gl_mtexable) //case 5: fullbright mask using multitexture
		{
			GL_DisableMultitexture(); // selects TEXTURE0
			GL_Bind (tx);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			GL_EnableMultitexture(); // selects TEXTURE1
			GL_Bind (fb);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
			glEnable(GL_BLEND);
			 if (r_int_animation.value)
				{
				GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
				}//Preach, I trust that using currententity makes this work
			else	
				GL_DrawAliasFrame (paliashdr, posenum);
			glDisable(GL_BLEND);
			GL_DisableMultitexture();
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		}
		else //case 6: fullbright mask without multitexture
		{
		// first pass
			GL_Bind(tx);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			 if (r_int_animation.value)
				{
				GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
				}//Preach, I trust that using currententity makes this work
			else	
				GL_DrawAliasFrame (paliashdr, posenum);
		// second pass
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			GL_Bind(fb);
			glEnable(GL_BLEND);
			glDepthMask(GL_FALSE);
			 if (r_int_animation.value)
				{
				GL_DrawAliasBlendedFrame (paliashdr, currententity->pose1, currententity->pose2, blend);
				}//Preach, I trust that using currententity makes this work
			else	
				GL_DrawAliasFrame (paliashdr, posenum);
			glDepthMask(GL_TRUE);
			glDisable(GL_BLEND);
		}
	}

cleanup:
	if (gl_smoothmodels.value && !r_drawflat_cheatsafe)
		glShadeModel (GL_FLAT);
	if (gl_affinemodels.value)
		glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	glPopMatrix ();

	if (r_shadows.value)
	{
		glDepthMask(GL_FALSE);
		GL_DrawAliasShadow (paliashdr, posenum);
		glDepthMask(GL_TRUE);
	}
}


/*
=============
GL_DrawAliasFrame_ShowTris -- johnfitz
=============
*/
void GL_DrawAliasFrame_ShowTris (aliashdr_t *paliashdr, int posenum)
{
	trivertx_t	*verts;
	int			*commands, count;

	verts = (trivertx_t *)((byte *)paliashdr + paliashdr->posedata);
	verts += posenum * paliashdr->poseverts;
	commands = (int *)((byte *)paliashdr + paliashdr->commands);

	while (1)
	{
		// get the vertex count and primitive type
		count = *commands++;
		if (!count)
			break;		// done

		if (count < 0)
		{
			count = -count;
			glBegin (GL_TRIANGLE_FAN);
		}
		else
			glBegin (GL_TRIANGLE_STRIP);

		do
		{
			commands += 2;
			glVertex3f (verts->v[0], verts->v[1], verts->v[2]);
			verts++;
		} while (--count);

		glEnd ();
	}
}

/*
=================
R_DrawAliasModel_ShowTris -- johnfitz
=================
*/
void R_DrawAliasModel_ShowTris (entity_t *e)
{
	aliashdr_t	*paliashdr;
	vec3_t		mins, maxs;
	int			posenum;

	VectorAdd (currententity->origin, currententity->model->mins, mins);
	VectorAdd (currententity->origin, currententity->model->maxs, maxs);
	if (R_CullBox (mins, maxs))
		return;

	VectorCopy (currententity->origin, r_entorigin);
	VectorSubtract (r_origin, r_entorigin, modelorg);

	paliashdr = (aliashdr_t *)Mod_Extradata (currententity->model);

    glPushMatrix ();
	R_RotateForEntity (e);
	glTranslatef (paliashdr->scale_origin[0], paliashdr->scale_origin[1], paliashdr->scale_origin[2]);
	glScalef (paliashdr->scale[0], paliashdr->scale[1], paliashdr->scale[2]);

	posenum = R_SetupAliasFrame (paliashdr, currententity->frame);

	GL_DrawAliasFrame_ShowTris (paliashdr, posenum);

	glPopMatrix ();
}